//
// (c) 2006 DS Data Systems UK Ltd, All rights reserved.
//
// DS Data Systems and KonaKart and their respective logos, are 
// trademarks of DS Data Systems UK Ltd. All rights reserved.
//
// The information in this document is free software; you can redistribute 
// it and/or modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
// 
// This software is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//

package com.konakart.bl.modules.payment.usaepay;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;

import com.konakart.app.KKConfiguration;
import com.konakart.app.KKException;
import com.konakart.app.NameValue;
import com.konakart.app.Order;
import com.konakart.app.OrderTotal;
import com.konakart.app.PaymentDetails;
import com.konakart.appif.KKEngIf;
import com.konakart.bl.modules.BaseModule;
import com.konakart.bl.modules.ordertotal.OrderTotalMgr;
import com.konakart.bl.modules.payment.BasePaymentModule;
import com.konakart.bl.modules.payment.PaymentInfo;
import com.konakart.bl.modules.payment.PaymentInterface;

/**
 * USA ePay payment module. This payment module allows for credit card credentials to be collected
 * directly from a KonaKart page. All communication to the USA ePay server is done from the KonaKart
 * server.
 * 
 */
public class Usaepay extends BasePaymentModule implements PaymentInterface
{

    // Module name must be the same as the class name although it can be all in lowercase in order
    // to remain compatible with osCommerce.
    private static String code = "usaepay";

    private static int sortOrder = -1;

    // The USA ePay Url used to POST the payment request.
    // "https://www.usaepay.com/gate.php "
    private static String usaePayRequestUrl;

    // The merchant key
    private static String usaePayKey;

    // Put the gateway in test mode
    private static boolean testMode = true;

    // Show the post code field on the UI
    private static boolean showPostcode = true;

    // Show the address field on the UI
    private static boolean showAddress = true;

    // Show the CVV field on the UI
    private static boolean showCVV = true;

    private static int zone;

    private static int orderStatusId;

    private static String bundleName = BaseModule.basePackage + ".payment.usaepay.Usaepay";

    private static HashMap<Locale, ResourceBundle> resourceBundleMap = new HashMap<Locale, ResourceBundle>();

    private static String mutex = "usaepayMutex";

    // Configuration Keys

    /**
     * Used to put the gateway online / offline
     */
    private final static String MODULE_PAYMENT_USAEPAY_STATUS = "MODULE_PAYMENT_USAEPAY_STATUS";

    /**
     * The UsaePayZone zone, if greater than zero, should reference a GeoZone. If the
     * DeliveryAddress of the order isn't within that GeoZone, then we throw an exception
     */
    private final static String MODULE_PAYMENT_USAEPAY_ZONE = "MODULE_PAYMENT_USAEPAY_ZONE";

    private final static String MODULE_PAYMENT_USAEPAY_ORDER_STATUS_ID = "MODULE_PAYMENT_USAEPAY_ORDER_STATUS_ID";

    /**
     * The order for displaying this payment gateway on the UI
     */
    private final static String MODULE_PAYMENT_USAEPAY_SORT_ORDER = "MODULE_PAYMENT_USAEPAY_SORT_ORDER";

    /**
     * To perform a transaction in test mode
     */
    private final static String MODULE_PAYMENT_USAEPAY_TESTMODE = "MODULE_PAYMENT_USAEPAY_TESTMODE";

    /**
     * To show post code field
     */
    private final static String MODULE_PAYMENT_USAEPAY_SHOW_POSTCODE = "MODULE_PAYMENT_USAEPAY_SHOW_POSTCODE";

    /**
     * To show address field
     */
    private final static String MODULE_PAYMENT_USAEPAY_SHOW_ADDRESS = "MODULE_PAYMENT_USAEPAY_SHOW_ADDRESS";

    /**
     * To show CVV field
     */
    private final static String MODULE_PAYMENT_USAEPAY_SHOW_CVV = "MODULE_PAYMENT_USAEPAY_SHOW_CVV";

    /**
     * The merchant key
     */
    private final static String MODULE_PAYMENT_USAEPAY_KEY = "MODULE_PAYMENT_USAEPAY_KEY";

    /**
     * The UsaePay Url used to POST the payment request. "https://www.usaepay.com/gate.php "
     */
    private final static String MODULE_PAYMENT_USAEPAY_REQUEST_URL = "MODULE_PAYMENT_USAEPAY_REQUEST_URL";

    // Message Catalogue Keys
    private final static String MODULE_PAYMENT_USAEPAY_TEXT_TITLE = "module.payment.usaepay.text.title";

    private final static String MODULE_PAYMENT_USAEPAY_TEXT_DESCRIPTION = "module.payment.usaepay.text.description";

    /**
     * Constructor
     * 
     * @param eng
     * 
     * @throws KKException
     */
    public Usaepay(KKEngIf eng) throws KKException
    {
        super.init(eng);

        // Initialise the static variables
        if (sortOrder == -1)
        {
            synchronized (mutex)
            {
                if (sortOrder == -1)
                {
                    setStaticVariables();
                }
            }
        }
    }

    /**
     * Sets some static variables during setup
     * 
     * @throws KKException
     * 
     */
    public void setStaticVariables() throws KKException
    {
        KKConfiguration conf;

        conf = getEng().getConfiguration(MODULE_PAYMENT_USAEPAY_KEY);
        if (conf == null)
        {
            throw new KKException(
                    "The Configuration MODULE_PAYMENT_USAEPAY_KEY must be set to the key"
                            + " released by USA ePay");
        }
        usaePayKey = conf.getValue();

        conf = getEng().getConfiguration(MODULE_PAYMENT_USAEPAY_REQUEST_URL);
        if (conf == null)
        {
            throw new KKException(
                    "The Configuration MODULE_PAYMENT_USAEPAY_REQUEST_URL must be set to the URL for"
                            + " sending the request to USA ePay. (i.e. https://www.usaepay.com/gate.php)");
        }
        usaePayRequestUrl = conf.getValue();

        conf = getEng().getConfiguration(MODULE_PAYMENT_USAEPAY_ZONE);
        if (conf == null)
        {
            zone = 0;
        } else
        {
            zone = new Integer(conf.getValue()).intValue();
        }

        conf = getEng().getConfiguration(MODULE_PAYMENT_USAEPAY_SORT_ORDER);
        if (conf == null)
        {
            sortOrder = 0;
        } else
        {
            sortOrder = new Integer(conf.getValue()).intValue();
        }

        conf = getEng().getConfiguration(MODULE_PAYMENT_USAEPAY_ORDER_STATUS_ID);
        if (conf == null)
        {
            orderStatusId = 0;
        } else
        {
            orderStatusId = new Integer(conf.getValue()).intValue();
        }

        conf = getEng().getConfiguration(MODULE_PAYMENT_USAEPAY_TESTMODE);
        if (conf == null)
        {
            testMode = true;
        } else
        {
            if (conf.getValue().trim().equalsIgnoreCase("false"))
            {
                testMode = false;
            } else
            {
                testMode = true;
            }
        }

        conf = getEng().getConfiguration(MODULE_PAYMENT_USAEPAY_SHOW_ADDRESS);
        if (conf == null)
        {
            showAddress = true;
        } else
        {
            if (conf.getValue().trim().equalsIgnoreCase("false"))
            {
                showAddress = false;
            } else
            {
                showAddress = true;
            }
        }

        conf = getEng().getConfiguration(MODULE_PAYMENT_USAEPAY_SHOW_POSTCODE);
        if (conf == null)
        {
            showPostcode = true;
        } else
        {
            if (conf.getValue().trim().equalsIgnoreCase("false"))
            {
                showPostcode = false;
            } else
            {
                showPostcode = true;
            }
        }

        conf = getEng().getConfiguration(MODULE_PAYMENT_USAEPAY_SHOW_CVV);
        if (conf == null)
        {
            showCVV = true;
        } else
        {
            if (conf.getValue().trim().equalsIgnoreCase("false"))
            {
                showCVV = false;
            } else
            {
                showCVV = true;
            }
        }

    }

    /**
     * Return a payment details object for USA ePay payment module
     * 
     * @param order
     * @param info
     * @return Returns information in a PaymentDetails object
     * @throws Exception
     */
    public PaymentDetails getPaymentDetails(Order order, PaymentInfo info) throws Exception
    {

        /*
         * The USA ePay zone, if greater than zero, should reference a GeoZone. If the
         * DeliveryAddress of the order isn't within that GeoZone, then we throw an exception
         */
        if (zone > 0)
        {
            checkZone(info, zone);
        }

        // Get the scale for currency calculations
        int scale = new Integer(order.getCurrency().getDecimalPlaces()).intValue();

        // Get the resource bundle
        ResourceBundle rb = getResourceBundle(mutex, bundleName, resourceBundleMap, info
                .getLocale());
        if (rb == null)
        {
            throw new KKException("A resource file cannot be found for the country "
                    + info.getLocale().getCountry());
        }

        PaymentDetails pDetails = new PaymentDetails();
        pDetails.setCode(code);
        pDetails.setSortOrder(sortOrder);
        pDetails.setOrderStatusId(orderStatusId);
        pDetails.setPaymentType(PaymentDetails.SERVER_PAYMENT_GATEWAY);
        pDetails.setDescription(rb.getString(MODULE_PAYMENT_USAEPAY_TEXT_DESCRIPTION));
        pDetails.setTitle(rb.getString(MODULE_PAYMENT_USAEPAY_TEXT_TITLE));

        // Return now if the full payment details aren't required. This happens when the manager
        // just wants a list of payment gateways to display in the UI.
        if (info.isReturnDetails() == false)
        {
            return pDetails;
        }

        pDetails.setPostOrGet("post");
        pDetails.setRequestUrl(usaePayRequestUrl);

        List<NameValue> parmList = new ArrayList<NameValue>();

        parmList.add(new NameValue("UMkey", usaePayKey));

        // USA ePay only requires details of the final price. No tax, subtotal etc.
        BigDecimal total = null;
        for (int i = 0; i < order.getOrderTotals().length; i++)
        {
            OrderTotal ot = (OrderTotal) order.getOrderTotals()[i];
            if (ot.getClassName().equals(OrderTotalMgr.ot_total))
            {
                total = ot.getValue().setScale(scale, BigDecimal.ROUND_HALF_UP);
            }
        }
      
        if (total == null)
        {
            throw new KKException("An Order Total was not found");
        }

        parmList.add(new NameValue("UMamount", total.toString()));
        parmList.add(new NameValue("UMinvoice", order.getId()));

        parmList.add(new NameValue("UMtestmode", (testMode ? "1" : "0")));
        parmList.add(new NameValue("UMcustid", order.getCustomerId()));
        parmList.add(new NameValue("UMcurrency", order.getCurrency().getCode()));

        // Set the billing name. If the name field consists of more than two strings, we take the
        // last one as being the surname.
        String bName = order.getBillingName();
        if (bName != null)
        {
            String[] names = bName.split(" ");
            int len = names.length;
            if (len >= 2)
            {
                StringBuffer firstName = new StringBuffer();
                for (int i = 0; i < len - 1; i++)
                {
                    if (firstName.length() == 0)
                    {
                        firstName.append(names[i]);
                    } else
                    {
                        firstName.append(" ");
                        firstName.append(names[i]);
                    }
                }
                parmList.add(new NameValue("UMbillfname", firstName.toString()));
                parmList.add(new NameValue("UMbilllname", names[len - 1]));
            }
        }

        // Set the billing address
        parmList.add(new NameValue("UMbillcompany", order.getCustomerCompany()));
        parmList.add(new NameValue("UMbillstreet", order.getBillingStreetAddress()));
        parmList.add(new NameValue("UMbillcity", order.getBillingCity()));
        parmList.add(new NameValue("UMbillstate", order.getBillingState()));
        parmList.add(new NameValue("UMbillzip", order.getBillingPostcode()));
        parmList.add(new NameValue("UMbillphone", order.getCustomerTelephone()));
        parmList.add(new NameValue("UMcustemail", order.getCustomerEmail()));
        parmList.add(new NameValue("UMbillcountry", order.getBillingCountry()));

        // Put the parameters into an array
        NameValue[] nvArray = new NameValue[parmList.size()];
        parmList.toArray(nvArray);
        pDetails.setParameters(nvArray);

        // Set the fields that should be visible in the UI when gathering Credit Card details
        pDetails.setShowAddr(showAddress);
        pDetails.setShowCVV(showCVV);
        pDetails.setShowPostcode(showPostcode);
        pDetails.setShowType(false); // USAePay doesn't require the card type

        if (log.isDebugEnabled())
        {
            log.debug(pDetails.toString());
        }

        return pDetails;
    }

    /**
     * Returns true or false
     * 
     * @throws KKException
     */
    public boolean isAvailable() throws KKException
    {
        return isAvailable(getEng(), MODULE_PAYMENT_USAEPAY_STATUS);
    }

}
